package com.example.fuzzControll.controller.testController;

import com.example.fuzzControll.exception.mysqlException.MysqlException;
import com.example.fuzzControll.exception.testException.AflnetException;
import com.example.fuzzControll.exception.threadException.MyExceptionHandler;
import com.example.fuzzControll.domain.vo.AjaxResult;
import com.example.fuzzControll.domain.bo.CmdStartParams;
import com.example.fuzzControll.service.TestService;
import com.example.fuzzControll.tools.system.GlobalClass;
import com.example.fuzzControll.tools.system.SystemRunningParams;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import java.util.LinkedHashMap;
import java.util.concurrent.ConcurrentHashMap;

/**
 * aflnet测试控制
 */
@Slf4j
@RestController
@RequestMapping("/test")
public class AlfnetController {
    @Autowired
    TestService service;

    /**
     * 测试启动
     */
    @RequestMapping(value = "/testStart", method = RequestMethod.POST)
    public AjaxResult start(@RequestBody final CmdStartParams cmdStartParams) {
        try {
            SystemRunningParams.aflnetData.put("missionName", cmdStartParams.getProtopcol());
            Thread subThread = new Thread(new Runnable() {
                @Override
                public void run() {
                    service.testStart(cmdStartParams);
                }
            });
            SystemRunningParams.ThreadState.put(subThread, "start");
            subThread.setUncaughtExceptionHandler(new MyExceptionHandler());
            subThread.start();
//            subThread.join();
            Thread.sleep(1000 * 10);//暂停3s,让系统运行至指令完全运行 todo跟据系统运行设定时间
            if ("error".equals(SystemRunningParams.ThreadState.get(subThread))) {//使用join串行就无法停止
                throw new Exception();
            }
        } catch (Exception e) {
            return AjaxResult.error("测试启动失败！");
        }
        return AjaxResult.success("测试已启动！");
    }

    /**
     * 测试停止
     */
    @RequestMapping(value = "/testStop", method = RequestMethod.GET)
    public AjaxResult testStop() {
        try {
            service.testStop();
        } catch (AflnetException e) {
            log.error(e.getDefaultMessage());
            return AjaxResult.error("测试停止失败！");
        } catch (MysqlException e) {
            log.error("File backup error: " + e.getMessage());
        }
        return AjaxResult.success("测试停止成功！");
    }

    /**
     * 获取测试信息
     */
    @RequestMapping(value = "/testProcessInfo", method = RequestMethod.GET)
    public AjaxResult getProcessInfo() {
        LinkedHashMap<String, String> result = new LinkedHashMap<String, String>();
        try {
            result = service.getProcessInfo();
        } catch (AflnetException e) {
            log.error(e.getDefaultMessage());
            return AjaxResult.error("测试信息获取失败！");
        }
        return AjaxResult.success(result);
    }
}

