package com.example.fuzzbackendmaster.service;

import com.example.fuzzbackendmaster.pojo.vo.*;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Component;
import org.springframework.stereotype.Service;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.List;


@FeignClient(value = "fuzz-backend-integration")
public interface FuzzIntegrationFileApi {
    /**
     * seedFlole
     */
    @RequestMapping(value = "/seedFile/list", method = RequestMethod.GET)
    AjaxResult list();

    @RequestMapping(value = "/seedFile/upload", method = RequestMethod.GET, consumes = MediaType.MULTIPART_FORM_DATA_VALUE)
    AjaxResult upload(@RequestPart("file") MultipartFile file);

    @RequestMapping(value = "/seedFile/delete", method = RequestMethod.GET, consumes = MediaType.MULTIPART_FORM_DATA_VALUE)
    AjaxResult delFile(@RequestPart("fileName") String fileName);

    /**
     * testclass
     */
    @RequestMapping(value = "/testClass/protocolTemplate", method = RequestMethod.POST)
    AjaxResult protocolTemplate(@RequestBody TestEntity testEntity);

    @RequestMapping(value = "/testClass/generate", method = RequestMethod.POST)
    AjaxResult generate(@RequestBody TestEntity testEntity);

    @RequestMapping(value = "/testClass/mutation", method = RequestMethod.POST)
    AjaxResult mutation(@RequestBody TestEntity testEntity);

    @RequestMapping(value = "/testClass/vulnerabilityType", method = RequestMethod.POST)
    AjaxResult vulnerability(@RequestBody TestEntity testEntity);

    /**
     * Aflnet
     */
    @RequestMapping(value = "/test/testStop", method = RequestMethod.GET)
    AjaxResult testStop();

    @RequestMapping(value = "/test/testStart", method = RequestMethod.POST)
    AjaxResult testStart(@RequestBody final CmdStartParams cmdStartParams);

    @RequestMapping(value = "/kittyServer/stats", method = RequestMethod.GET)
    AjaxResult getStats();

    @RequestMapping(value = "/kittyServer/templateInfo", method = RequestMethod.GET)
    AjaxResult getTemplateInfo();

    @RequestMapping(value = "/kittyServer/stages", method = RequestMethod.GET)
    AjaxResult getStages();

    @RequestMapping(value = "/kittyServer/report", method = RequestMethod.GET)
    AjaxResult getReport();

    /**
     * websocket
     */
    @RequestMapping(value = "/websocket/connect", method = RequestMethod.GET)
    void connect();

    @RequestMapping(value = "/websocket/disConnect", method = RequestMethod.GET)
    void disconnect();

    @RequestMapping(value = "/log/download/{missionId}", method = RequestMethod.GET)
    List<FuzzLogTransEntity> getFuzzLog(@PathVariable("missionId") int missionId);

    /**
     * 查询任务信息
     *
     * @return
     */
    @RequestMapping(value = "/mission/getList", method = RequestMethod.GET)
    AjaxResult selectMissionInfoList();

    /**
     * 查询测试参数
     */
    @RequestMapping(value = "/fuzzParams/list", method = RequestMethod.GET)
    AjaxResult getFuzzParamsList();

    /**
     * 跟据id查参数
     *
     * @param missionId
     * @return
     */
    @RequestMapping(value = "/fuzzParams/getParam/{missionId}", method = RequestMethod.GET)
    AjaxResult getFuzzParamById(@PathVariable("missionId") int missionId);

    /**
     * 编辑参数
     *
     * @param fuzzParams
     * @return
     */
    @RequestMapping(value = "/fuzzParams/editParam", method = RequestMethod.POST)
    AjaxResult editFuzzParams(FuzzParams fuzzParams);

    /**
     * 获取测试过程信息
     */
    @RequestMapping(value = "/test/testProcessInfo", method = RequestMethod.GET)
    AjaxResult getProcessInfo();

    /**
     * 开启网络代理
     */
    @RequestMapping(value = "/agent/kittyNetworkAgent", method = RequestMethod.GET)
    AjaxResult startNetworkAgent(@RequestParam String networkCard, @RequestParam String monitorName);

    /**
     * 获取代理生成的pcap文件
     */
    @RequestMapping(value = "/agent/getAgentPcap", method = RequestMethod.GET)
    byte[] getAgentPcap();

    /**
     * 开启ssh代理
     */
    @RequestMapping(value = "/agent/kittySshAgent", method = RequestMethod.GET)
    AjaxResult startSshAgent(@RequestParam String MonitorName, @RequestParam String SshUserName,
                             @RequestParam String SshPassword, @RequestParam String SshIp,
                             @RequestParam String SshPort, @RequestParam String Command);

    /**
     * 异常重放分析
     */
    @RequestMapping(value = "/aflnet/analyse", method = RequestMethod.GET)
    AjaxResult analyse(@RequestParam String codePath,@RequestParam String programName);

    /**
     * 获取异常分析结果
     */
    @RequestMapping(value = "/aflnet/getreplayresult", method = RequestMethod.GET)
    AjaxResult getReplayResult();
    /**
     * 异常重放
     */
    @RequestMapping(value = "/aflnet/replay", method = RequestMethod.GET)
    AjaxResult replay(@RequestParam String targetServer, @RequestParam String targetServerCodePath,
                      @RequestParam String targetServerPort, @RequestParam String protocol,
                      @RequestParam String pathToCrash);
}
